#!/usr/bin/env python
# jobcontrol.py

"""
Author : peters 

Revision History:
File created on 06 Sep 2010.

"""

import sys
import os
import logging
import subprocess

std_joboptions={'jobname':'test','jobaccount':'co2','jobnodes':'nserial 1','jobshell':'/bin/sh','depends':'','jobtime':'00:30:00'}

class PlatForm(object):
    """ This specifies platform dependent options under generic object calls. A platform object is used to control and submit jobs"""

    def __init__(self):
        self.Identifier     = 'iPad'    # the identifier gives the plaform name
        self.Version        = '1.0'     # the platform version used

        msg1  = '%s object initialized'%self.Identifier ; logging.debug(msg1)
        msg2  = '%s version: %s'%(self.Identifier,self.Version) ; logging.debug(msg2)


    def __str__(self):

        return None

    def GetJobTemplate(self,joboptions={}):
        """ Return the job template for a given computing system, and fill it with options from the dictionary provided as argument"""

        template = """## \n"""+ \
                   """## This is a set of dummy names, to be replaced by values from the dictionary \n"""+ \
                   """## Please make your own platform specific template with your own keys and place it in a subfolder of the da package.\n """+ \
                   """## \n"""+ \
                   """ \n"""+ \
                   """#$ jobname \n"""+ \
                   """#$ jobaccount \n"""+ \
                   """#$ jobnodes \n"""+ \
                   """#$ jobtime \n"""+ \
                   """#$ jobshell \n """

        if 'depends' in joboptions:
            template += """#$ -hold_jid depends \n"""

        # First replace from passed dictionary
        for k,v in joboptions.iteritems():
            while k in template:
                template = template.replace(k,v)

        # Fill remaining values with std_options
        for k,v in std_joboptions.iteritems():
            while k in template:
                template = template.replace(k,v)

        return template

    def GetMyID(self):

        return os.getpid()

    def WriteJob(self,DaCycle,template, jobid):                   
        """ This method writes a jobfile to the exec dir"""
        #
        # Done, write jobfile
        # 
        targetdir               = os.path.join(DaCycle.da_settings['dir.exec'])
        jobfile                 = os.path.join(targetdir,'jb.%s.jb'%jobid)
        f                       = open(jobfile,'w')
        dummy                   = f.write(template)
        dummy                   = f.close()
        dummy                   = os.chmod(jobfile,477)                  

        msg = "A job file was created (%s)"%jobfile  ; logging.debug(msg)

        return jobfile
        

    def SubmitJob(self,jobfile):                   
        """ This method submits a jobfile to the queue, and returns the queue ID """
        
        cmd     = ['sh',jobfile]
        msg = "A new task will be started (%s)\n\n\n"%cmd  ; logging.debug(msg)
        process = subprocess.Popen(cmd)
        jobid   = subprocess.Popen(cmd).pid
        dummy   = subprocess.Popen(cmd).wait()
         
        return 0

    def KillJob(self,jobid):                   
        """ This method kills a running job """
        
        return None

    def StatJob(self,jobid):                   
        """ This method gets the status of a running job """
        import subprocess
        
        output = subprocess.Popen(['qstat',jobid], stdout=subprocess.PIPE).communicate()[0]  ; logging.info(output)

        return output


if __name__ == "__main__":
    pass
